package ch.tvbauma.opencms.module.guestbook;

import com.opencms.file.*;
import com.opencms.core.*;
import com.opencms.util.*;
import com.opencms.template.*;
import com.opencms.workplace.*;
import com.opencms.defaults.*;

import java.util.*;
import java.io.*;
import java.sql.*;
import java.text.*;

/**
 * <p>Title: Guestbook</p>
     * <p>Description: Little helper building a questbook, storing entries in DB</p>
 * <p>Copyright: Freeware - NO WARRANTY</p>
 * <p>Company: Gymnastics Association Bauma, Switzerland</p>
 * @author Cyrill Brunschwiler - BRU
 * @version 1.0
 */
public class GuestbookBackoffice
    extends A_CmsBackoffice {

  // setup some static values
  private static Locale m_locale = new Locale("Default", "" + Locale.getDefault());

  /**
   * Provides getting this class
   * @return The class its self
   */
  public Class getContentDefinitionClass() {
    return GuestbookContentDefinition.class;
  }

  public String getCreateUrl(CmsObject cms, String tagcontent,
                             A_CmsXmlContent doc, Object userObject) throws
      Exception {
    return "system/modules/ch.tvbauma.opencms.module.guestbook/"
        + "administration/guestbook/EditBackoffice.html";
  }

  public String getBackofficeUrl(CmsObject cms, String tagcontent,
                                 A_CmsXmlContent doc, Object userObject) throws
      Exception {
    return "system/modules/ch.tvbauma.opencms.module.guestbook/" +
        "administration/guestbook/Backoffice.html";
  }

  public String getEditUrl(CmsObject cms, String tagcontent,
                           A_CmsXmlContent doc, Object userObject) throws
      Exception {
    return "system/modules/ch.tvbauma.opencms.module.guestbook/" +
        "administration/guestbook/EditBackoffice.html";
  }

  public byte[] getContentEdit(CmsObject cms,
                               CmsXmlWpTemplateFile template,
                               String elementName,
                               Hashtable parameters,
                               String templateSelector) throws CmsException {
    String error = "";

    CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
    String prefix = "ch_tvbauma_opencms_module_guestbook_GuestbookBackoffice.";
    String smileypath = lang.getLanguageValue(prefix + "path.smileys");

    // session will be created or fetched
    I_CmsSession session =
        (CmsSession) cms.getRequestContext().getSession(true);

    // get request params
    String action = (String) parameters.get("action");
    String id = (String) parameters.get("id");

    // validate input
    if (action == null)
      action = "";

    if (id == null)
      id = "";

    if (action.equals("save")) {
      //move to the done section in the template
      templateSelector = "done";

      //get data from the template
      int iId = Integer.valueOf(id).intValue();

      String sAuthor = (String) parameters.get("author");
      String sText = (String) parameters.get("text");
      String sEmail = (String) parameters.get("email");
      String sLink = (String) parameters.get("link");

      GuestbookContentDefinition editCD = GuestbookContentDefinition.getEntry(
          iId);

      // clear params
      if (sAuthor == null)
        sAuthor = "";

      if (sText == null)
        sText = "";

      if (sEmail == null)
        sEmail = "";

      if (sLink == null || "http://".equals(sLink))
        sLink = "";

      // get patterns for input validation
      prefix = "ch_tvbauma_opencms_module_guestbook_GuestbookBackoffice.pattern.";

      String patternAuthor = lang.getLanguageValue(prefix + "name");
      String patternText = lang.getLanguageValue(prefix + "text");
      String patternEmail = lang.getLanguageValue(prefix + "email");
      String patternLink = lang.getLanguageValue(prefix + "link");

      String messageAuthor = lang.getLanguageValue(prefix + "name.message");
      String messageText = lang.getLanguageValue(prefix + "text.message");
      String messageEmail = lang.getLanguageValue(prefix + "email.message");
      String messageLink = lang.getLanguageValue(prefix + "link.message");
      String messageSmiley = lang.getLanguageValue(prefix + "tag.message");
      String messageInvalidPattern = lang.getLanguageValue(prefix +
          "invalid.message");


      // input validation
      boolean isInputValidated = false;

      try {
        isInputValidated = ("".equals(sAuthor) ||
                            Util.check(sAuthor, patternAuthor)) &&
            ("".equals(sText) || Util.check(sText, patternText) || Util.checkSmileyTags(sText, cms)) &&
            ("".equals(sEmail) || Util.check(sEmail, patternEmail)) &&
            ("".equals(sLink) || Util.check(sLink, patternLink));

        if (! ("".equals(sAuthor) || Util.check(sAuthor, patternAuthor)))
          error += messageAuthor + "<br>";

        if (! ("".equals(sText) || Util.check(sText, patternText))) {
          error += messageText + "<br>";
        }
        else {
          if (!Util.checkSmileyTags(sText, cms))
            error += messageSmiley;
        }

        if (! ("".equals(sEmail) || Util.check(sEmail, patternEmail)))
          error += messageEmail + "<br>";

        if (! ("".equals(sLink) || Util.check(sLink, patternLink)))
          error += messageLink + "<br>";
      }
      catch (Exception ex) {
        // isInputValidated is still false... ok do nothing
        error += messageInvalidPattern + "<br>";
      }

      // send form again if any wrong inputs were detected
      if ("".equals(sAuthor) || "".equals(sText) || !"".equals(error)) {
        templateSelector = "default";
        template.setData("author", sAuthor);
        template.setData("text", sText);
        template.setData("email", sEmail);
        template.setData("link", sLink);
        template.setData("date", editCD.getFormatedDate());
        template.setData("time", editCD.getFormatedTime());

        // colorize required fields
        if (sText.equals("")) {
          template.setData("textHighlightStart", template.getData("redStart"));
          template.setData("textHighlightEnd", template.getData("redEnd"));
        }

        // colorize required fields
        if (sAuthor.equals("")) {
          template.setData("authorHighlightStart", template.getData("redStart"));
          template.setData("authorHighlightEnd", template.getData("redEnd"));
        }

        try {
          String smileys = Util.getAllSmileysAsHTML(cms, smileypath);

          if (smileys != null) {
            template.setData("smileysbegin", template.getData("tablesmileybegin"));
            template.setData("smileys", smileys);
            template.setData("smileysend", template.getData("tablesmileyend"));
          }
        }
        catch (Exception ex) {
          // shouldn't happen... don't display smileys
        }

        template.setData("error", error);
        template.setData("setaction", "save"); // still not first time shown
        return startProcessing(cms, template, elementName, parameters,
                               templateSelector);
      }
      //change the cd object content
      editCD.setAuthor(sAuthor);
      editCD.setText(sText);
      editCD.setEmail(sEmail);
      editCD.setLink(sLink);

      try {
        editCD.write(cms);
      }
      catch (Exception e) {
        System.err.println("GuestbookEditBackoffice: couldnt update!");
        System.err.println("[" + this.getClass().getName() + "] " +
                           e.getMessage());
        template.setData("error", e.toString());
        template.setData("author", sAuthor);
        template.setData("text", sText);
        template.setData("email", sEmail);
        template.setData("link", sLink);
        template.setData("date", editCD.getFormatedDate());
        template.setData("time", editCD.getFormatedTime());
        template.setData("setaction", "save");

        // insert smileys
        try {
          String smileys = Util.getAllSmileysAsHTML(cms, smileypath);

          if (smileys != null) {
            template.setData("smileysbegin", template.getData("tablesmileybegin"));
            template.setData("smileys", smileys);
            template.setData("smileysend", template.getData("tablesmileyend"));
          }
        }
        catch (Exception ex) {
          // shouldn't happen... don't display smileys
        }

        templateSelector = "default";
        return startProcessing(cms, template, elementName, parameters,
                               templateSelector);
      }
    }

    if (id != null && !id.equals("")) {

      //get data from cd object
      int iId = Integer.valueOf(id).intValue();
      GuestbookContentDefinition gcd = GuestbookContentDefinition.getEntry(iId);

      //set the data from the CD in the template
      template.setData("author", gcd.getAuthor());
      template.setData("text", gcd.getText());
      template.setData("email", gcd.getEmail());
      template.setData("link", gcd.getLink());
      template.setData("date", gcd.getFormatedDate());
      template.setData("time", gcd.getFormatedTime());

      try {
        String smileys = Util.getAllSmileysAsHTML(cms, smileypath);

        if (smileys != null) {
          template.setData("smileysbegin", template.getData("tablesmileybegin"));
          template.setData("smileys", smileys);
          template.setData("smileysend", template.getData("tablesmileyend"));
        }
      }
      catch (Exception ex) {
        // shouldn't happen... don't display smileys
      }

      template.setData("setaction", "save");

      // re-display data in selectbox
      return startProcessing(cms, template, elementName, parameters,
                             templateSelector);
    }
    //finally start the processing
    return startProcessing(cms, template, elementName, parameters,
                           templateSelector);
  }

  /**
   * Called if a new guestbook entry will be created in the backoffice
   *
   * @param cms The cms object
   * @param template The template object
   * @param elementName The name of the element
   * @param parameters The requests parameters as a hash table
   * @param templateSelector The selected part of the template
   *
   * @throws CmsException If an arror occures while processing the content
   */
  public byte[] getContentNew(CmsObject cms,
                              CmsXmlWpTemplateFile template,
                              String elementName,
                              Hashtable parameters,
                              String templateSelector) throws CmsException {

    // session will be created or fetched
    I_CmsSession session = (CmsSession) cms.getRequestContext().getSession(true);

    // setup values
    String error = "";
    CmsXmlLanguageFile lang = new CmsXmlLanguageFile(cms);
    String prefix = "ch_tvbauma_opencms_module_guestbook_GuestbookBackoffice.";
    String smileypath = lang.getLanguageValue(prefix + "path.smileys");


    // get params from request
    String action = (String) parameters.get("action");
    String id = (String) parameters.get("id");

    // validate input
    if (action == null)
      action = "";

    if (id == null)
      id = "";

      // check for param action
    if (action == null || action.equals("")) {
      // action unknown

      templateSelector = "default";
      template.setData("setaction", "default");

      // set actual Date
      template.setData("date", GuestbookContentDefinition.getActualDate());
      template.setData("time", GuestbookContentDefinition.getActualTime());
      template.setData("link", "http://");

      // build current users full name and set it for the template
      String firstname = cms.getRequestContext().currentUser().getFirstname();
      String lastname = cms.getRequestContext().currentUser().getLastname();
      String name = cms.getRequestContext().currentUser().getName();

      // get smileys
      try {
        String smileys = Util.getAllSmileysAsHTML(cms, smileypath);

        if (smileys != null) {
          template.setData("smileysbegin", template.getData("tablesmileybegin"));
          template.setData("smileys", smileys);
          template.setData("smileysend", template.getData("tablesmileyend"));
        }
      }
      catch (Exception ex) {
        // shouldn't happen... don't display smileys
      }

      if ( (firstname != null && !firstname.equals("")) &&
          (lastname != null && !lastname.equals(""))) {
        template.setData("author", firstname + " " + lastname);
      }
      else if (lastname != null && !lastname.equals("")) {
        template.setData("author", lastname);
      }
      else {
        template.setData("author", name);
      }
    }
    else {
      // action ok, create new data.. create new content definition instance
      templateSelector = "done";

      //read value of the inputfields
      String sAuthor = (String) parameters.get("author");
      String sText = (String) parameters.get("text");
      String sEmail = (String) parameters.get("email");
      String sLink = (String) parameters.get("link");

      // clear params
      if (sAuthor == null)
        sAuthor = "";

      if (sText == null)
        sText = "";

      if (sEmail == null)
        sEmail = "";

      if (sLink == null || "http://".equals(sLink))
        sLink = "";

        // get patterns for input validation
      lang = new CmsXmlLanguageFile(cms);
      prefix = "ch_tvbauma_opencms_module_guestbook_GuestbookBackoffice.pattern.";

      String patternAuthor = lang.getLanguageValue(prefix + "name");
      String patternText = lang.getLanguageValue(prefix + "text");
      String patternEmail = lang.getLanguageValue(prefix + "email");
      String patternLink = lang.getLanguageValue(prefix + "link");

      String messageAuthor = lang.getLanguageValue(prefix + "name.message");
      String messageText = lang.getLanguageValue(prefix + "text.message");
      String messageEmail = lang.getLanguageValue(prefix + "email.message");
      String messageLink = lang.getLanguageValue(prefix + "link.message");
      String messageInvalidPattern = lang.getLanguageValue(prefix +
          "invalid.message");

      // input validation
      boolean isInputValidated = false;

      try {
        isInputValidated = ("".equals(sAuthor) ||
                            Util.check(sAuthor, patternAuthor)) &&
            ("".equals(sText) || Util.check(sText, patternText)) &&
            ("".equals(sEmail) || Util.check(sEmail, patternEmail)) &&
            ("".equals(sLink) || Util.check(sLink, patternLink));

        if (! ("".equals(sAuthor) || Util.check(sAuthor, patternAuthor)))
          error += messageAuthor + "<br>";

        if (! ("".equals(sText) || Util.check(sText, patternText)))
          error += messageText + "<br>";

        if (! ("".equals(sEmail) || Util.check(sEmail, patternEmail)))
          error += messageEmail + "<br>";

        if (! ("".equals(sLink) || Util.check(sLink, patternLink)))
          error += messageLink + "<br>";
      }
      catch (Exception ex) {
        // isInputValidated is still false... ok do nothing
        error += messageInvalidPattern + "<br>";
      }

      // send form again if any wrong inputs were detected
      if (!isInputValidated || "".equals(sAuthor) || "".equals(sText)) {
        templateSelector = "default";
        template.setData("author", sAuthor);
        template.setData("text", sText);
        template.setData("email", sEmail);
        template.setData("link", sLink);
        template.setData("date", GuestbookContentDefinition.getActualDate());
        template.setData("time", GuestbookContentDefinition.getActualTime());

        // colorize required fields
        if (sText.equals("")) {
          template.setData("textHighlightStart", template.getData("redStart"));
          template.setData("textHighlightEnd", template.getData("redEnd"));
        }

        // colorize required fields
        if (sAuthor.equals("")) {
          template.setData("authorHighlightStart", template.getData("redStart"));
          template.setData("authorHighlightEnd", template.getData("redEnd"));
        }

        try {
          String smileys = Util.getAllSmileysAsHTML(cms, smileypath);

          if (smileys != null) {
            template.setData("smileysbegin", template.getData("tablesmileybegin"));
            template.setData("smileys", smileys);
            template.setData("smileysend", template.getData("tablesmileyend"));
          }
        }
        catch (Exception ex) {
          // shouldn't happen... don't display smileys
        }

        template.setData("error", error);
        template.setData("setaction", "save"); // still not first time shown
        return startProcessing(cms, template, elementName, parameters,
                               templateSelector);
      }

      //create new cd object with the fetched values of the inputfields
      GuestbookContentDefinition newCD = new GuestbookContentDefinition(sAuthor,
          sText, sEmail, sLink);

      try {
        newCD.write(cms);
      }
      catch (Exception e) {
        System.err.println("GuestbookEditBackoffice: couldnt write!");
        System.err.println("[" + this.getClass().getName() + "] " +
                           e.getMessage());
        template.setData("error", e.toString());
        template.setData("author", sAuthor);
        template.setData("text", sText);
        template.setData("email", sEmail);
        template.setData("link", sLink);
        template.setData("date", GuestbookContentDefinition.getActualDate());
        template.setData("time", GuestbookContentDefinition.getActualTime());
        template.setData("setaction", "save");

        try {
          String smileys = Util.getAllSmileysAsHTML(cms, smileypath);

          if (smileys != null) {
            template.setData("smileysbegin", template.getData("tablesmileybegin"));
            template.setData("smileys", smileys);
            template.setData("smileysend", template.getData("tablesmileyend"));
          }
        }
        catch (Exception ex) {
          // shouldn't happen... don't display smileys
        }

        templateSelector = "default";
      }
    }

    //finally start the processing
    return startProcessing(cms, template, elementName, parameters,
                           templateSelector);
  }
}